{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: AVI files - playing and exporting to HTML }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}


unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, RVScroll, RichView, RVEdit, RVStyle, MPlayer, ExtCtrls,
  RVItem, CRVData, RVTypes;

type
  TForm1 = class(TForm)
    RVStyle1: TRVStyle;
    RichViewEdit1: TRichViewEdit;
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    Button2: TButton;
    SaveDialog1: TSaveDialog;
    procedure Button1Click(Sender: TObject);
    procedure RichViewEdit1ControlAction(Sender: TCustomRichView;
      ControlAction: TRVControlAction; ItemNo: Integer;
      var ctrl: TControl);
    procedure Button2Click(Sender: TObject);
    procedure RichViewEdit1SaveComponentToFile(Sender: TCustomRichView;
      Path: TRVUnicodeString; SaveMe: TPersistent; SaveFormat: TRVSaveFormat;
      var OutStr: TRVUnicodeString);
    procedure RichViewEdit1ItemAction(Sender: TCustomRichView;
      ItemAction: TRVItemAction; Item: TCustomRVItemInfo;
      var Text: TRVUnicodeString; RVData: TCustomRVData);
  private
    { Private declarations }
    procedure OnPlayerResize(Sender: TObject);
    procedure OnPlayerClick(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
{------------------------------------------------------------------------------}
{ Inserting new video 200x200 }
procedure TForm1.Button1Click(Sender: TObject);
var Panel, VideoPanel: TPanel;
    Player: TMediaPlayer;
begin
  if not OpenDialog1.Execute then
    exit;

  // Creating the player panel
  Panel := TPanel.Create(nil);
  Panel.BevelInner := bvNone;
  Panel.BevelOuter := bvNone;
  Panel.Width := 200;

  // Creating a panel to show video
  VideoPanel := TPanel.Create(Panel);
  VideoPanel.Parent := Panel;
  VideoPanel.BevelInner := bvNone;
  VideoPanel.BevelOuter := bvNone;
  VideoPanel.Width := 200;
  VideoPanel.Height := 200;
  VideoPanel.Color := clBlack;

  // Creating a player control
  Player := TMediaPlayer.Create(Panel);
  Player.Parent := Panel;
  Player.ColoredButtons := [];
  Player.VisibleButtons := [btPlay,btPause,btPrev];
  Player.Height := 20;
  Panel.Height := VideoPanel.Height+Player.Height;
  Player.Left := (Panel.Width-{Player.Width}85) div 2;
  Player.Top  := VideoPanel.Height;
  Player.Display := VideoPanel;
  VideoPanel.Hint := ExtractFileName(OpenDialog1.FileName);
  VideoPanel.ShowHint := True;

  Panel.OnResize := OnPlayerResize;
  Panel.OnClick := OnPlayerClick;
  VideoPanel.OnClick := OnPlayerClick;

  // Inserting in TRichViewEdit
  RichViewEdit1.TopLevelEditor.BeginUndoGroup(rvutInsert);
  RichViewEdit1.TopLevelEditor.SetUndoGroupMode(True); // grouping InsertControl & SetCurrentItemExtraIntProperty
  try
    if RichViewEdit1.InsertControl('', Panel, rvvaBaseline) then begin
      RichViewEdit1.SetCurrentItemExtraIntProperty(rvepResizable, 1, True);
      Player.FileName := OpenDialog1.FileName;
      Player.Open;
    end;
  finally
    RichViewEdit1.TopLevelEditor.SetUndoGroupMode(False);
  end;
end;
{------------------------------------------------------------------------------}
{ Saving to HTML }
procedure TForm1.Button2Click(Sender: TObject);
begin
  if SaveDialog1.Execute then
    RichViewEdit1.SaveHTMLEx(SaveDialog1.FileName, 'Demo', 'img', '', '', '', []);
end;
{------------------------------------------------------------------------------}
{ Selecting on click }
procedure TForm1.OnPlayerClick(Sender: TObject);
var Panel: TPanel;
begin
  Panel := Sender as TPanel;
  if Panel.Parent is TPanel then
    Panel := TPanel(Panel.Parent);
  RichViewEdit1.SelectControl(Panel)
end;
{------------------------------------------------------------------------------}
{ Smart resizing }
procedure TForm1.OnPlayerResize(Sender: TObject);
var Panel, VideoPanel: TPanel;
    Player: TMediaPlayer;
begin
  Panel      := Sender as TPanel;
  VideoPanel := Panel.Components[0] as TPanel;
  Player     := Panel.Components[1] as TMediaPlayer;
  VideoPanel.Width := Panel.Width;
  VideoPanel.Height := Panel.Height-{Player.Height}20;
  Player.Left := (Panel.Width-{Player.Width}85) div 2;
  Player.Top  := VideoPanel.Height;
end;
{------------------------------------------------------------------------------}
{ Event: OnControlAction }
procedure TForm1.RichViewEdit1ControlAction(Sender: TCustomRichView;
  ControlAction: TRVControlAction; ItemNo: Integer; var ctrl: TControl);
var Panel, VideoPanel: TPanel;
    Player: TMediaPlayer;
begin
  if not (Ctrl is TPanel) then
    exit;
  Panel      := Ctrl as TPanel;
  VideoPanel := Panel.Components[0] as TPanel;
  Player     := Panel.Components[1] as TMediaPlayer;
  case ControlAction of
    rvcaMoveToUndoList:
      begin
        // Moving from editor to undo buffer: closing video
        Panel.Visible := False;
        Panel.Parent := RichViewEdit1;
        Player.Close;
        Panel.Parent := nil;
      end;
    rvcaMoveFromUndoList:
      begin
        // Moving from undo buffer back to editor: opening video
        Player.Open;
      end;
    rvcaAfterRVFLoad:
      begin
        // Loading from RVF file or stream: assigning Display and events, opening video
        Player.Display := VideoPanel;
        Player.Open;
        Panel.OnClick := OnPlayerClick;
        VideoPanel.OnClick := OnPlayerClick;
        Panel.OnResize := OnPlayerResize;
      end;
  end;
end;
{------------------------------------------------------------------------------}
{ Event: OnItemAction }
procedure TForm1.RichViewEdit1ItemAction(Sender: TCustomRichView;
  ItemAction: TRVItemAction; Item: TCustomRVItemInfo;
  var Text: TRVUnicodeString; RVData: TCustomRVData);
var Panel, VideoPanel: TPanel;
    Player: TMediaPlayer;
begin
  if csDestroying in Sender.ComponentState then
    exit;
  if (ItemAction=rviaInserted) and (Item.StyleNo=rvsComponent) and
    (TRVControlItemInfo(Item).Control is TPanel) then
  begin
    Panel      := TRVControlItemInfo(Item).Control as TPanel;
    VideoPanel := Panel.Components[0] as TPanel;
    Player     := Panel.Components[1] as TMediaPlayer;
    Player.Display := VideoPanel;
  end;
end;
{------------------------------------------------------------------------------}
{ Event: OnSaveComponentToFile }
procedure TForm1.RichViewEdit1SaveComponentToFile(Sender: TCustomRichView;
  Path: TRVUnicodeString; SaveMe: TPersistent; SaveFormat: TRVSaveFormat;
  var OutStr: TRVUnicodeString);
var
  Panel: TPanel;
  Player: TMediaPlayer;
  FileName: TRVUnicodeString;
begin
  // Saving as <object>
  if (SaveMe is TPanel) and (SaveFormat=rvsfHTML) then
  begin
    Panel      := SaveMe as TPanel;
    Player     := Panel.Components[1] as TMediaPlayer;
    FileName := ExtractRelativePath(Path, Player.FileName);
    FileName := StringReplace(FileName, '\', '/', [rfReplaceAll]);
    OutStr := Format(
     '<object type="video/avi" width="%d" height="%d">'+
       '<param name="filename" value="%s" />'+
       '<a href="%s">Video</a>'+
     '</object>',
      [Panel.Width, Panel.Height, FileName, FileName]);
  end;
end;


initialization
  RegisterClasses([TPanel, TMediaPlayer]);

end.
